# $Id: mod_poldek.py,v 1.1.1.1 2002/04/28 09:16:50 mis Exp $
# mirrortool module to configure PLD poldek's sources

import xreadlines
import ConfigParser
import re
import os
import string


pld_dist_dir      = "dists/%dist/PLD/%arch/PLD/RPMS/"
pld_dist_testdir  = "dists/%dist/test/%arch/"

pld_source_list = [ 'pld', 'pld-test', 'nest', 'nest-test' ]
pld_source_conf = {
    'pld':        { 'dist': 'ra',  'uri': pld_dist_dir,
                    'opts': '' },
    
    'pld-test':   { 'dist': 'ra',  'uri': pld_dist_testdir,
                    'opts': 'noauto,noautoup' },
    
    'nest':       { 'dist': 'nest', 'uri': pld_dist_dir,
                    'opts': 'noauto,noautoup', },
    
    'nest-test':  { 'dist': 'nest', 'uri': pld_dist_testdir,
                    'opts': 'noauto,noautoup', },
}

schema_conf = { 'PLD': { 'source_list' : pld_source_list,
                         'source_conf' : pld_source_conf },
}
    

def mirrortool_summary():
    return "configure poldek sources"

def mirrortool_options():
    return """conf=CONFIG - use poldek config CONFIG; default is ~/.poldekrc
                  or /etc/poldek.conf)
    out=FILE - write result to FILE; default is CONFIG, previous version
               is saved as CONFIG-

    schema=SCHEMA - configuration schema, available schemas %s (default is %s)
    """ % (schema_conf.keys(), 'PLD')

def expand_dir(dir, dist):
    arch = os.uname()[4]
    expdir = string.replace(dir, "%dist", dist)
    expdir = string.replace(expdir, "%arch", arch)
    return expdir


def open_file(fn):
    try:
        fp = open(fn)
    except:
        return None
    return fp


def open_config(fn = None):
    fp = None
    if not fn:
        fn = os.environ['HOME'] + '/.poldekrc'
        
    try:
        fp = open(fn)
    except:
        fn = '/etc/poldek.conf'
        try:
            fp = open(fn)
        except IOError, e:
            print "%s: %s" % (e.filename, e.strerror)
            raise
        
    return fp
    


def mirrortool_configure(site_name, site_url, schema = 'PLD',
                         conf = None, out = None):
    fp = open_config(conf)
    
    lines = []
    existing_sources = {}
    source_lineno = -1
    i = 0

    try:
        source_conf = schema_conf[schema]['source_conf']
        source_list = schema_conf[schema]['source_list']
    except:
        print "%s: no such schema" % schema
        raise
        
    for line in xreadlines.xreadlines(fp):
        if source_lineno == -1 and re.match('^\s*source\s*=', line):
            source_lineno = i
            
        m = re.match('^\s*source\s*=\s*([\w-]+),?(.*?)\s+\w+', line)
        if m and source_conf.has_key(m.group(1)):
            existing_sources[m.group(1)] = m.group(2)
            line = "#" + line;    
                
        lines.append(line)
        i += 1

    if not out:
        out = fp.name
    fp.close()
    
    if os.access(out, os.R_OK | os.W_OK):
        os.rename(out, out + '-');
        
    try:
        fp = open(out, "w")
    except IOError, e:
        print "%s: %s" % (e.filename, e.strerror)
        raise
        
    for i in range(0, source_lineno - 1):
        fp.write(lines[i])

    fp.write("\n# %s\n" % site_name)     
    for nam in source_list:
        srconf = source_conf[nam]
        source_url = site_url + '/' + expand_dir(srconf['uri'], srconf['dist'])
        opts = srconf['opts']
        if existing_sources.has_key(nam):
            opts = existing_sources[nam]
                
        if len(opts) == 0:
            opts = nam
        else:
            opts = nam + "," + opts

        line = "source = %s %s\n" % (opts, source_url)
        fp.write(line)
        #print line,
            
    fp.write("\n")
        
    for i in range(source_lineno - 1, len(lines)):
        fp.write(lines[i])
            
    fp.close()

    print "Saved %s" % fp.name


if __name__ == '__main__':
    try:      
        configure('ftp.dupa.com', 'http://foo.org')
    
    except Exception, e:
        print "\n\nDEBUG TRACEBACK"
        raise
        
